package net.bajobongo.zerro.parkowanie;

import net.bajobongo.zerro.geometry.*;
import net.bajobongo.zerro.geometry.Polygon;
import java.awt.*;
import java.lang.Math;

public class Car implements Cloneable {
	public Position front;
	double carDirection = Math.PI;
	public double wheelDirection = 0;
	double length = 100;
	public double velocity = 0;
	double wheelMax = (Math.PI/4.0);
	double width = 50;
	double wheel_length = 30;
	double wheel_width_padding = 5;

	public Car(Position front, double carDirection, int width, int length) {
		this.front = front;
		this.carDirection = carDirection;
		this.width = width;
		this.length = length;
		this.wheel_length = length / 4;
	}
	public Polygon polygon() {
		Position carNE = Geometry.move(front, width/2.0, carDirection + (Math.PI/2.0));
		Position carNW = Geometry.move(front, width/2.0, carDirection - (Math.PI/2.0));
		Position carSE = Geometry.move(carNE, length, carDirection + Math.PI);
		Position carSW = Geometry.move(carNW, length, carDirection + Math.PI);
		return new Polygon(
			new int[] {(int)carNE.x, (int)carNW.x, (int)carSE.x, (int)carSW.x},
			new int[] {(int)carNE.y, (int)carNW.y, (int)carSE.y, (int)carSW.y},
			4
		);
	}
	public void draw(Graphics g) {
//		Position back = Geometry.move(front, length, carDirection + Math.PI);
//		Position arrow = Geometry.move(front, 10, carDirection + wheelDirection);
		Position carNE = Geometry.move(front, width/2.0, carDirection + (Math.PI/2.0));
		Position carNW = Geometry.move(front, width/2.0, carDirection - (Math.PI/2.0));
		Position carSE = Geometry.move(carNE, length, carDirection + Math.PI);
		Position carSW = Geometry.move(carNW, length, carDirection + Math.PI);
		g.setColor(Color.black);
		g.drawLine((int)carNE.x, (int)carNE.y, (int)carSE.x, (int)carSE.y);
		g.drawLine((int)carSE.x, (int)carSE.y, (int)carSW.x, (int)carSW.y);
		g.drawLine((int)carSW.x, (int)carSW.y, (int)carNW.x, (int)carNW.y);
		g.drawLine((int)carNW.x, (int)carNW.y, (int)carNE.x, (int)carNE.y);
		Position SEWheelBack = Geometry.move(carSE, wheel_width_padding, carDirection - (Math.PI/2.0));
		Position SEWheelFront = Geometry.move(SEWheelBack, wheel_length, carDirection);
		Position SWWheelBack = Geometry.move(carSW, wheel_width_padding, carDirection + (Math.PI/2.0));
		Position SWWheelFront = Geometry.move(SWWheelBack, wheel_length, carDirection);
		Position NEWheel = Geometry.move(
			Geometry.move(carNE, wheel_width_padding, carDirection - (Math.PI/2.0)),
			wheel_length/2.0,
			carDirection + Math.PI
		);
		Position NEWheelFront = Geometry.move(NEWheel, wheel_length/2.0, carDirection + wheelDirection);
		Position NEWheelBack = Geometry.move(NEWheel, wheel_length/2.0, carDirection + wheelDirection + Math.PI);
		Position NWWheel = Geometry.move(
			Geometry.move(carNW, wheel_width_padding, carDirection + (Math.PI/2.0)),
			wheel_length/2.0,
			carDirection - Math.PI
		);
		Position NWWheelFront = Geometry.move(NWWheel, wheel_length/2.0, carDirection + wheelDirection);
		Position NWWheelBack = Geometry.move(NWWheel, wheel_length/2.0, carDirection + wheelDirection + Math.PI);
		g.setColor(Color.red);
		g.drawLine((int)SEWheelBack.x, (int)SEWheelBack.y, (int)SEWheelFront.x, (int)SEWheelFront.y);
		g.drawLine((int)SWWheelBack.x, (int)SWWheelBack.y, (int)SWWheelFront.x, (int)SWWheelFront.y);
		g.drawLine((int)NEWheelFront.x, (int)NEWheelFront.y, (int)NEWheelBack.x, (int)NEWheelBack.y);
		g.drawLine((int)NWWheelFront.x, (int)NWWheelFront.y, (int)NWWheelBack.x, (int)NWWheelBack.y);
	}
	public void turn(double angle) {
		wheelDirection -= angle;
		wheelDirection = (wheelDirection <= wheelMax) ? wheelDirection : wheelMax;
		wheelDirection = (wheelDirection >= -wheelMax) ? wheelDirection : -wheelMax;
	}
	public void step(double frequency) {
		double distance = velocity / frequency;
		move(distance);
	}
	public void move(double distance) {
		Position newFront = Geometry.move(front, distance, carDirection + wheelDirection);
		Position back = Geometry.move(front, length, carDirection + Math.PI);
		Position newFrontTranslated = new Position(newFront.x - back.x, newFront.y - back.y);
		Position newFrontRotated = Geometry.rotate(newFrontTranslated, -carDirection);
		double distanceOfNewFrontFromOldCarAxis = newFrontRotated.x;
		double p = Math.sqrt(length*length - distanceOfNewFrontFromOldCarAxis*distanceOfNewFrontFromOldCarAxis);
		double backDistance = Geometry.distance(back, newFront) - p;
		Position newBack = Geometry.move(back, backDistance, carDirection);
		double newDirection = Geometry.direction(newBack, newFront);
		front = newFront;
		carDirection = newDirection;
	}
	public Object clone() {
		try {
			return super.clone();
		} catch (CloneNotSupportedException e) {
			return null;
		}
	} 
}


