package net.bajobongo.zerro.parkowanie;

import net.bajobongo.zerro.geometry.*;
import net.bajobongo.zerro.geometry.Polygon;
import net.bajobongo.zerro.plansza.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.Math;

public class Parkowanie extends Plansza {
	Poziom[] poziomy;
	Poziom poziom;
	int numerPoziomu = 0;
	Car car;
	boolean steering = false;
	boolean initialized = false;
	int mouseCenterX = 0;
	int mouseCenterY = 0;
	double startWheelDirection = 0;
	double velocitySensitivity = (300.0/300.0);
	double wheelSensitivity = (Math.PI/300);
	GameStatus gameStatus = GameStatus.PLAYING;
	Guzik guzik;

	class NextLevelGuzik extends Guzik {
		public NextLevelGuzik() {
			super(0, 0, 70, 20, "next level");
		}
		public void pressedAction(MouseEvent e) {
			nextLevel();
			System.out.println("jestem guzikiem nextlevel");
		}
	}

	class RestartLevelGuzik extends Guzik {
		public RestartLevelGuzik() {
			super(0, 0, 80, 20, "restart level");
		}
		public void pressedAction(MouseEvent e) {
			restartLevel();
			System.out.println("jestem guzikiem restartlevel");
		}
	}

	public void keyPressed( KeyEvent e ) { 
		if (e.getKeyCode() == KeyEvent.VK_LEFT) {
			car.turn(-0.1);
		}
		if (e.getKeyCode() == KeyEvent.VK_RIGHT) {
			car.turn(0.1);
		}
		if (e.getKeyCode() == KeyEvent.VK_UP) {
			car.velocity++;
		}
		if (e.getKeyCode() == KeyEvent.VK_DOWN) {
			car.velocity--;
		}
	}

	public void mouseReleased (MouseEvent e) { 
		steering = false;
		car.velocity = 0;
	}
	public void mousePressed (MouseEvent e) { 
		if (guzik != null) guzik.pressed(e);
		steering = true;
		mouseCenterX = e.getX();
		mouseCenterY = e.getY();
		startWheelDirection = car.wheelDirection;
	}

	public void mouseDragged(MouseEvent e) { 
		car.velocity = -(e.getY() - mouseCenterY) * velocitySensitivity;
		car.wheelDirection = startWheelDirection + (-(e.getX() - mouseCenterX) * wheelSensitivity);
		car.wheelDirection = (car.wheelDirection <= car.wheelMax) ? car.wheelDirection : car.wheelMax;
		car.wheelDirection = (car.wheelDirection >= -car.wheelMax) ? car.wheelDirection : -car.wheelMax;
	}

	public void checkGameStatus() {
		if (gameStatus != GameStatus.LEVEL_LOST && poziom.target.containsPolygon(car.polygon())) {
			gameStatus = GameStatus.LEVEL_WON;
			poziom.targetColor = Color.blue;
			guzik = new NextLevelGuzik();
		}
		for (int i = 0; i < poziom.obstacles.length; i++) {
			if (poziom.obstacles[i].intersectsPolygon(car.polygon())) {
				gameStatus = GameStatus.LEVEL_LOST;
				poziom.obstaclesColor = Color.red;
				guzik = new RestartLevelGuzik();
			}
		}
	}

	public void paintFrame(Graphics g) {
		if (car != null) {
			checkGameStatus();
			car.step(frequency);
			car.draw(g);
			poziom.draw(g);
			if (guzik!= null) guzik.draw(g);
			//offGraphics.setColor(Color.blue);
			//g.drawString("kierownica: " + car.wheelDirection, 4, 10);
			//g.drawString("samochod: " + car.carDirection, 4, 30);
			//g.drawString("predkosc: " + car.velocity, 4, 50);
		}
	}

	void restartLevel() {
		poziom = poziomy[numerPoziomu];
		car = (Car) poziom.car.clone();
		gameStatus = GameStatus.PLAYING;
		poziom.obstaclesColor = Color.black;
		guzik = null;
	}

	void nextLevel() {
		++numerPoziomu;
		if (numerPoziomu == poziomy.length) numerPoziomu = 0;
		restartLevel();
	}

	public void init() {
		poziomy = new PoziomyReader("levels").poziomy;
		restartLevel();
		super.init();
	}
}
