package net.bajobongo.zerro.parkowanie;

import java.util.*;
import net.bajobongo.zerro.geometry.*;
import net.bajobongo.zerro.geometry.Polygon;
import net.bajobongo.zerro.utils.*;

class PoziomReader extends FileLineByLine {
	public Poziom poziom;
	protected double SCALE=20;
	protected ArrayList obstacles = new ArrayList();
	protected Polygon target = new Polygon();
	protected Position carFront;
	protected int carWidth;
	protected int carHeight;
	protected double carDirection;
	protected int extractAttributeInt(String line, String attributeName) {
		int result = 0;
		String attribute = line.split(attributeName + "=\"")[1].split("\"")[0];
		result = (int) (Double.parseDouble(attribute) * SCALE);
		return(result);
	}
	protected double extractAttributeDouble(String line, String attributeName) {
		return(Double.parseDouble(line.split(attributeName + "=\"")[1].split("\"")[0]) * SCALE);
	}
	protected PolygonWithProperties processPolygonLine(String line) {
		String color = line.split("stroke: #")[1].split("\"")[0];
		String[] points = line.split("points=\"")[1].split("\"")[0].split(" ");
		int[] x = new int[points.length];
		int[] y = new int[points.length];
		int n;
		for (n = 0; n < points.length; n++) {
			String[] coords = points[n].split(",");
			x[n] = (int) (Double.parseDouble(coords[0]) * SCALE);
			y[n] = (int) (Double.parseDouble(coords[1]) * SCALE);
		}
		Polygon polygon = new Polygon(x, y, n);
		return(new PolygonWithProperties(polygon, color));
	}
	protected PolygonWithProperties processRectangleLine(String line) {
		String color = line.split("stroke: #")[1].split("\"")[0];
		int x = extractAttributeInt(line, "x");
		int y = extractAttributeInt(line, "y");
		int width  = extractAttributeInt(line, "width");
		int height = extractAttributeInt(line, "height");
		Polygon polygon = new Polygon(
			new int[] {x, x + width, x + width,  x         },
			new int[] {y, y,         y + height, y + height},
			4
		);
		PolygonWithProperties p = new PolygonWithProperties(polygon, color);
		p.x = x;
		p.y = y;
		p.width = width;
		p.height = height;
		return(p);
	}
	protected void processLine(String line) {
		PolygonWithProperties polygonWithProperties = null;
		Polygon polygon = null;
		String color = null;
		if (line.matches(".*polygon .*")) {
			polygonWithProperties = processPolygonLine(line);
			polygon = polygonWithProperties.polygon;
			color = polygonWithProperties.color;
		}
		if (line.matches(".*rect .*")) {
			polygonWithProperties = processRectangleLine(line);
			polygon = polygonWithProperties.polygon;
			color = polygonWithProperties.color;
		}
		if (polygon != null) {
			if (color.equals("000000")) {
				obstacles.add(polygon);
			}
			else if (color.equals("00ff00")) {
				target = polygon;
			}
			else if (color.equals("ff0000")) {
				carWidth = polygonWithProperties.width;
				carHeight = polygonWithProperties.height;
			}
		}
		if (line.matches(".*line .*") && line.matches(".*stroke: #ff0000.*")) {
			int x1 = extractAttributeInt(line, "x1");
			int y1 = extractAttributeInt(line, "y1");
			int x2 = extractAttributeInt(line, "x2");
			int y2 = extractAttributeInt(line, "y2");
			carFront = new Position(x1, y1);
			carDirection = Geometry.direction(x1, y1, x2, y2);
		}
	}
	protected void afterLastLine() { 
		Car car = new Car(carFront, carDirection, carWidth, carHeight);
		poziom = new Poziom(car, target, (Polygon[]) obstacles.toArray(new Polygon[0]));
	}
	public PoziomReader(String filename) {
		super(filename);
		process();
	}
}
